/**************************************************************************
**************************************************************************
************
************		AIX Agent
************
**************************************************************************
**************************************************************************/

/**************************************************************************/
/*	Includes and Class Usage Declarations
*/

#include 	<dirent.h>
#include 	<stdio.h>
#include 	<time.h>

#ifdef	Unix_Test			/* test defines and prototypes */

	#include 	<dos.h>

	#include 	"TestSupport.h"

     #define	FUNCTION_sleep		_sleep
     #define	FUNCTION_stricmp	stricmp

#else					/* actual defines and prototypes */

     #include  <errno.h>
	#include 	<fstab.h>
     #include  <libperfstat.h>
	#include 	<sys/procfs.h>
	#include	<sys/utsname.h>
	#include 	<unistd.h>

     #define	FUNCTION_sleep		sleep
     #define	FUNCTION_stricmp	strcasecmp

#endif


/**************************************************************************/
/* 	Defines and Typedef Information
*/

#define	AGENT_version					3

#define	VALUE_endian 					0x2468ace0

#define	FILEEXT_max					10

#define	INFOTYPE_interval_record			0
#define	INFOTYPE_differ_record			1

#define	RECID_group_start				0xfefefa00
#define	RECID_group_baseline    			0xfefefb00
#define	RECID_group_interval    			0xfefefc00
#define	RECID_group_differ	    			0xfefefd00
#define	RECID_group_time				0xfefefe00
#define	RECID_group_end				0xfefefeff

#define	RECID_data_proc				0x11
#define	RECID_data_file_system			0x12
#define	RECID_data_disk				0x13
#define	RECID_data_memory				0x14
#define	RECID_data_network				0x15
#define	RECID_data_processor_total		0x16
#define	RECID_data_processor 			0x17

#define	RECID_value(a,b)				(a | b)


typedef 	int (*bsearch_compare) (const void*, const void*);

typedef	struct {
	long		size,
			agent_version,			/* agent version number 	*/
     		date,				/* yyyymmdd 			*/
               time;				/* seconds     		*/
     time_t 	base_time;
     struct tm	local_time;
     char		platform_type[20];
	long     	int_size,
               endian_check,
     		sizeof_fstab,
               sizeof_lwpinfo,
               sizeof_psinfo,
               sizeof_lwpstatus,
               sizeof_pstatus,
               sizeof_perfstat_cpu_total,
               sizeof_perfstat_cpu,
               sizeof_perfstat_disk,
               sizeof_perfstat_memory_total,
               sizeof_perfstat_netinterface,
     		system_name_size,
               node_name_size,
               release_size,
               version_size,
               machine_size,
               timezone_shift_seconds;
	} RECORD_File_Start;

typedef	struct {
	long		date,		    	/* yyyymmdd						*/
               time,		    	/* seconds     					*/
               interval_seconds;  	/* number of seconds in the interval 	*/
	} RECORD_Info_Record;

/* 	Proc Table structures and defines */

typedef	struct {
	psinfo_t	info;   			/* psinfo structure information 		*/
     pstatus_t	status;			/* pstatus structure information		*/
	} PROCTABLE_Data;

typedef	struct {
	char		active;			/* see if the process is still active	*/
	uint64_t	id;   			/* proc id 						*/
     PROCTABLE_Data
     		*data;			/* data							*/
	} PROCTABLE_Entry;


/**************************************************************************/
/* 	Static Information
*/

			/* processing options 	*/
char		option_midnight_command[300],
		option_file_ext[50];
long		option_interval_minutes,
		option_review_seconds;

               /* globals			*/
DIR		*directory;
FILE		*file;
FILE      *diag;
PROCTABLE_Entry
		*proc_table;
struct tm	last_time,
		current_date,
		current_interval,
          next_interval;
char		file_name[300],
		*disk_map_logical,
		*disk_map_physical,
     	*dm_log_ptr,
     	*dm_phy_ptr;
long		interval_length,
    		next_is_review,
          proc_table_inuse,
          proc_table_limit,
		review_length,
          timer_interval;


/**************************************************************************/
/* 	Support Functions
*/

struct tm Get_Now (void) {
time_t 	timer;
	timer = time (NULL);
   	return (*localtime (&timer));
	}

long Calculate_Time_Shift (struct tm gm_time, struct tm local_time) {
long		gm_seconds,
          local_seconds;
	if (gm_time.tm_year != local_time.tm_year)
     	if (gm_time.tm_year < local_time.tm_year) {
          	local_time.tm_yday++;
               gm_time.tm_yday = 0;
          	}
		   else {
          	gm_time.tm_yday++;
               local_time.tm_yday = 0;
			}
     gm_seconds = (gm_time.tm_yday * 86400) + (gm_time.tm_hour * 3600) +
     	(gm_time.tm_min * 60) + gm_time.tm_sec;
     local_seconds = (local_time.tm_yday * 86400) + (local_time.tm_hour * 3600) +
     	(local_time.tm_min * 60) + local_time.tm_sec;
     return (gm_seconds - local_seconds);     
	}

long Get_Seconds (struct tm time) {
     return ((time.tm_hour * 3600) + (time.tm_min * 60) + time.tm_sec);
     }

long Get_Julian_Date (struct tm time) {
	return (((time.tm_year + 1900) * 1000) + time.tm_yday + 1);
     }

struct tm Set_Time (long seconds) {
struct tm time;
     memset (&time, 0x00, sizeof (time));
     time.tm_hour = seconds / 3600;
     time.tm_min = (seconds % 3600) / 60;
     time.tm_sec = seconds % 60;
     return (time);
     }

void Create_Intervals (struct tm time) {
long		seconds,
		adj_seconds;
     interval_length = option_interval_minutes * 60;
     seconds = (time.tm_hour * 3600) + (time.tm_min * 60) + time.tm_sec;
	adj_seconds = (seconds % interval_length);
     seconds -= adj_seconds;
     current_interval = Set_Time (seconds);
	seconds += interval_length;
     if (seconds >= 86400)
     	seconds = 86399;
     next_interval = Set_Time (seconds);
     if (adj_seconds > 20)
	     interval_length -= adj_seconds;
	}

long Set_Timer (void) {
struct tm time,
		next_time;
long		seconds,
          next_review_time;
     time = Get_Now ();
     if (! next_is_review)
	     Create_Intervals (time);
     review_length = option_review_seconds;
     seconds = (time.tm_hour * 3600) + (time.tm_min * 60) + time.tm_sec;
	next_review_time = seconds + (review_length - (seconds % review_length));
     if (next_review_time >= 86400)
     	next_review_time = 0;
     next_time = Set_Time (next_review_time);
     next_is_review = (Get_Seconds (next_time) < Get_Seconds (next_interval));
     time = Get_Now ();
     seconds = (time.tm_hour * 3600) + (time.tm_min * 60) + time.tm_sec;
     timer_interval = review_length - (seconds % review_length);
     return (1);
	}


/**************************************************************************/
/*	Options Functions
*/

long Get_Option_Entries (char **in_source, char *attribute, char *text) {
char		*ptr,
		*source = *in_source,
          t_attr[100],
          t_text[500];
long		length;
    	while ((*source > 0) && (*source < ' '))
     	source++;
     if (! *source) return (0);
	ptr = strchr (source, ':');
     if (! ptr) return (0);
     length = ptr - source;
     memcpy (t_attr, source, length);
     *(t_attr+length) = 0;
     source = ptr + 1;
    	while (*ptr >= ' ')
         	ptr++;
     length = ptr - source;
     memcpy (t_text, source, length);
     *(t_text+length) = 0;
     *in_source = ptr;
     ptr = t_attr + strlen (t_attr) - 1;
     while (*ptr == ' ')
     	*ptr-- = 0;
     ptr = t_attr;
     while (*ptr == ' ')
     	ptr++;
	strcpy (attribute, ptr);
     ptr = t_text + strlen (t_text) - 1;
     while (*ptr == ' ')
     	*ptr-- = 0;
     ptr = t_text;
     while (*ptr == ' ')
     	ptr++;
	strcpy (text, ptr);
	return (1);
	}

void Read_Options (void) {
char      attribute[100],
          text[300],
          data[2000];
FILE		*in_file;
char		*ptr;
long		size;
	in_file = fopen ("AgentOptions.txt", "r");
     if (! in_file) return;
     size = fread (data, sizeof (data)-1, 1, in_file);
	data[size] = 0;
     fclose (in_file);
     ptr = data;
     while (Get_Option_Entries (&ptr, attribute, text)) {
     	if (! FUNCTION_stricmp (attribute, "Midnight Shell Command")) {
          	if (*text == '\"') {
	          	strcpy (option_midnight_command, text+1);
                    *(option_midnight_command+strlen(option_midnight_command)-1) = 0;
                    }
                  else strcpy (option_midnight_command, text);
          	continue;
               }
     	if (! FUNCTION_stricmp (attribute, "File Extension")) {
          	strcpy (option_file_ext, text);
          	continue;
               }
     	if (! FUNCTION_stricmp (attribute, "Interval Minutes")) {
          	option_interval_minutes = atol (text);
          	continue;
               }
     	if (! FUNCTION_stricmp (attribute, "Review Seconds")) {
          	option_review_seconds = atol (text);
          	continue;
               }
     	}
	}


/**************************************************************************/
/* 	PerfStat Functions
*/

long Output_PerfStat_Disk (long group_id) {
perfstat_id_t
		name;
perfstat_disk_t
		rec[10];
int		count;
long		val;
	strcpy (name.name, "");
	count = perfstat_disk (&name, rec, sizeof (rec[0]), 10);
     if (count <= 0) return (1);
	val = RECID_value (group_id, RECID_data_disk);
     fwrite (&val, sizeof (long), 1, file);
     while (1) {
		fwrite (&rec, sizeof (rec[0]), count, file);
     	if (! *name.name) break;
		count = perfstat_disk (&name, rec, sizeof (rec[0]), 10);
     	}
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     return (1);
	}

long Output_PerfStat_Memory (long group_id) {
perfstat_memory_total_t
		rec;
long		val;
	if (perfstat_memory_total ((perfstat_id_t*) NULL, &rec, sizeof (rec), 1) <= 0) return (1);
	val = RECID_value (group_id, RECID_data_memory);
     fwrite (&val, sizeof (long), 1, file);
	fwrite (&rec, sizeof (rec), 1, file);
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     return (1);
	}

long Output_PerfStat_Network (long group_id) {
perfstat_id_t
		name;
perfstat_netinterface_t
		rec[10];
int		count;
long		val;
	strcpy (name.name, "");
	count = perfstat_netinterface (&name, rec, sizeof (rec[0]), 10);
     if (count <= 0) return (1);
	val = RECID_value (group_id, RECID_data_network);
     fwrite (&val, sizeof (long), 1, file);
     while (1) {
		fwrite (&rec, sizeof (rec[0]), count, file);
     	if (! *name.name) break;
		count = perfstat_netinterface (&name, rec, sizeof (rec[0]), 10);
     	}
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     return (1);
	}

long Output_PerfStat_Processor_Total (long group_id) {
perfstat_id_t
		name;
perfstat_cpu_total_t
		rec;
int		count;
long		val;
	count = perfstat_cpu_total (NULL, &rec, sizeof (rec), 1);
     if (count <= 0) return (1);
	val = RECID_value (group_id, RECID_data_processor_total);
     fwrite (&val, sizeof (long), 1, file);
	fwrite (&rec, sizeof (rec), 1, file);
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     return (1);
	}

long Output_PerfStat_Processor (long group_id) {
perfstat_id_t
		name;
perfstat_cpu_t
		rec[10];
int		count;
long		val;
	strcpy (name.name, "");
	count = perfstat_cpu (&name, rec, sizeof (rec[0]), 10);
     if (count <= 0) return (1);
	val = RECID_value (group_id, RECID_data_processor);
     fwrite (&val, sizeof (long), 1, file);
     while (1) {
		fwrite (&rec, sizeof (rec[0]), count, file);
     	if (! *name.name) break;
		count = perfstat_cpu (&name, rec, sizeof (rec[0]), 10);
     	}
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     return (1);
	}


/**************************************************************************/
/* 	File System Functions
*/

long Output_File_System_Data (long group_id) {
struct fstab
		*fs;
long		got_first = 0,
		val;
	setfsent ();
	while (1) {
     	fs = getfsent ();
          if (! fs) break;
          if (! got_first) {
          	got_first = 1;
			val = RECID_value (group_id, RECID_data_file_system);
		     fwrite (&val, sizeof (long), 1, file);
          	}
          fwrite (fs, sizeof (struct fstab), 1, file);
     	}
     endfsent ();
     if (got_first) {
     	val = RECID_group_end;
	     fwrite (&val, sizeof (long), 1, file);
     	}
     return (1);
	}


/**************************************************************************/
/* 	Process Table Functions
*/

void Initialize_Process_Table (void) {
	proc_table_inuse = 0;
	proc_table_limit = 1000;
	proc_table = (PROCTABLE_Entry*) calloc (proc_table_limit, sizeof (PROCTABLE_Entry));
	}

void End_Process_Table (void) {
long		i;
	for (i=0; i<proc_table_inuse; i++)
     	free (proc_table[i].data);
     free (proc_table);
	}

int Proc_Table_Compare (const PROCTABLE_Entry *p1, const PROCTABLE_Entry *p2) {
   	return (p1->id - p2->id);
	}

long Find_Proc_Table_Index (uint64_t id) {
long		i,
		index;
	for (i=0; i<proc_table_inuse; i++) {
     	if (proc_table[i].id > id) return (-1);
     	if (proc_table[i].id == id)
          	return (i);
          }
     return (-1);
     }

PROCTABLE_Entry* Find_Proc_Table_Entry (uint64_t id) {
PROCTABLE_Entry
		key,
          *entry;
	key.id = id;
	entry = (PROCTABLE_Entry*) bsearch (&key, proc_table, proc_table_inuse,
     	sizeof(PROCTABLE_Entry), (bsearch_compare) Proc_Table_Compare);
	return (entry);
    }

PROCTABLE_Entry* Add_Proc_Table_Entry (uint64_t id) {
PROCTABLE_Entry
		*entry;
long		i;
	entry = Find_Proc_Table_Entry (id);
     if (entry) return (entry);
     if (proc_table_inuse >= proc_table_limit) {
     	proc_table_limit += 1000;
          proc_table = (PROCTABLE_Entry*) realloc (
               proc_table, proc_table_limit * sizeof (PROCTABLE_Entry));
          }
	for (i=proc_table_inuse; i>=0; i--) {
     	if (id > proc_table[i-1].id) break;
          proc_table[i] = proc_table[i-1];
          }
     if (i < 0)
     	i = 0;     
     proc_table[i].active = 0;
	proc_table[i].id = id;
     proc_table[i].data = (PROCTABLE_Data*) calloc (1, sizeof (PROCTABLE_Data));
     proc_table_inuse++;
     return (&proc_table[i]);
     }

void Clear_Proc_Table_Active (void) {
long		i;
	for (i=0; i<proc_table_inuse; i++)
     	proc_table[i].active = 0;
     }

void Delete_Proc_Table_Entries (void) {
long		i,
		new_i = 0;
	for (i=0; i<proc_table_inuse; i++) {
     	if (proc_table[i].active) {
          	proc_table[new_i++] = proc_table[i];
               continue;
               }
	     free (proc_table[i].data);
          }
     proc_table_inuse = new_i;
	}


/**************************************************************************/
/* 	/proc Functions
*/

uint64_t Get_Next_Proc_Id (void) {
struct dirent
		*directory_entry;
     while (1) {
		errno = 0;
	    	directory_entry = readdir (directory);
		if (! directory_entry) {
          	if (errno)
               	abort ();
          	break;
               }
     	if (! isdigit (*directory_entry->d_name)) continue;
          return (atoll (directory_entry->d_name));
     	}
     return (-1);
     }

PROCTABLE_Entry* Add_New_Proc_Table_Entry (uint64_t id) {
PROCTABLE_Entry
		*entry;
FILE		*in_file;
psinfo_t  rec;
char		in_file_name[100];
     sprintf (in_file_name, "/proc/%d/psinfo", id);
     in_file = fopen (in_file_name, "r");
     if (! in_file) return (0);

     fread (&rec, sizeof (psinfo_t), 1, in_file);
     fwrite (&rec, sizeof (psinfo_t), 1, diag);

 	entry = Add_Proc_Table_Entry (id);
     entry->active = 1;
     fseek (in_file, 0, SEEK_SET);
     fread (&entry->data->info, sizeof (psinfo_t), 1, in_file);
     fclose (in_file);
	return (entry);
     }
     
void Update_Proc_Information (long differ) {
PROCTABLE_Entry
		*entry;
FILE		*in_file;
char		in_file_name[100];
long		id;
    	rewinddir (directory);
	id = Get_Next_Proc_Id ();
     while (id >= 0) {
          entry = Find_Proc_Table_Entry (id);
          if (entry)
          	if (entry->active) goto next_entry;
          if (! entry)
			entry = Add_New_Proc_Table_Entry (id);
/*          sprintf (in_file_name, "/proc/%d/status", id);
          in_file = fopen (in_file_name, "r");
          if (! in_file) goto next_entry;
          if (entry) {
          	entry->active = 1;
	          fread (&entry->data->status, sizeof (pstatus_t), 1, in_file);
     	     fclose (in_file);
          	}
*/
next_entry:
		id = Get_Next_Proc_Id ();
     	}
	}

long Output_Proc_Data (long entry_type, long only_inactive_records) {
long		i,
          got_first = 0;
     if (! entry_type) got_first = 1;     
	for (i=0; i<proc_table_inuse; i++) {
     	if (proc_table[i].active)
          	if (only_inactive_records) continue;
		if (! got_first) {
          	got_first = 1;
		     fwrite (&entry_type, sizeof (long), 1, file);
			}
          fwrite (&proc_table[i].data->info, sizeof (psinfo_t), 1, file);
          fwrite (&proc_table[i].data->status, sizeof (pstatus_t), 1, file);
          }
     return (got_first);     
	}


/**************************************************************************/
/* 	Information Functions
*/

void Store_Baseline_Information (void) {
long		val;
			/* perfstat	*/
	Output_PerfStat_Disk (RECID_group_baseline);
	Output_PerfStat_Memory (RECID_group_baseline);
	Output_PerfStat_Network (RECID_group_baseline);
	Output_PerfStat_Processor_Total (RECID_group_baseline);
	Output_PerfStat_Processor (RECID_group_baseline);
			/* file system 	*/
     Output_File_System_Data (RECID_group_baseline);
			/* proc	*/
	val = RECID_value (RECID_group_baseline, RECID_data_proc);
     fwrite (&val, sizeof (long), 1, file);
	Clear_Proc_Table_Active ();
     Update_Proc_Information (0);
     Output_Proc_Data (0, 0);
     Delete_Proc_Table_Entries ();
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
	}

long Store_Information (long info_type) {
struct tm	now;
RECORD_Info_Record
		record;
long		val,
		result;
     record.date = ((current_date.tm_year + 1900) * 10000) +
     	((current_date.tm_mon + 1) * 100) + current_date.tm_mday;
     record.time = (current_interval.tm_hour * 3600) + (current_interval.tm_min * 60) +
     	current_interval.tm_sec;
     record.interval_seconds = interval_length;
     if (info_type == INFOTYPE_differ_record) {
     	now = Get_Now ();
	     val = (now.tm_hour * 3600) + (now.tm_min * 60) + now.tm_sec;
          record.interval_seconds = val - record.time;
     	}
	val = RECID_group_time;
     fwrite (&val, sizeof (long), 1, file);
	fwrite (&record, sizeof (RECORD_Info_Record), 1, file);
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
	if (info_type == INFOTYPE_differ_record) goto do_proc;
			/* perfstat 	*/
	Output_PerfStat_Disk (RECID_group_interval);
	Output_PerfStat_Memory (RECID_group_interval);
	Output_PerfStat_Network (RECID_group_interval);
	Output_PerfStat_Processor_Total (RECID_group_baseline);
	Output_PerfStat_Processor (RECID_group_baseline);
			/* file system 	*/
     Output_File_System_Data (RECID_group_interval);
do_proc:
	val = (info_type == INFOTYPE_interval_record) ?
     		RECID_value (RECID_group_interval, RECID_data_proc) :
               RECID_value (RECID_group_differ, RECID_data_proc);
	Clear_Proc_Table_Active ();
     Update_Proc_Information ((info_type == INFOTYPE_differ_record) ? 1 : 0);
     result = Output_Proc_Data (val, (info_type == INFOTYPE_differ_record) ? 1 : 0);
     Delete_Proc_Table_Entries ();
     if (result) {
		val = RECID_group_end;
     	fwrite (&val, sizeof (long), 1, file);
          }
	fflush (file);
	return (1);
	}

     
/**************************************************************************/
/* 	Starting Functions
*/

void Create_File_Extension (void) {
struct utsname
		name;
char		new_name[300],
		*pnew,
          *pname;
long		count;
     strcpy (option_file_ext, "res");
     uname (&name);
     pname = name.sysname;
	count = strlen (pname);
     pnew = new_name;
     while (count--) {
     	if (isalnum (*pname))
          	*pnew++ = *pname;
          pname++;
          }
     *pnew = 0;
     count = strlen (new_name);
  	if (count < 3) return;
     pnew = new_name;
     if (count > FILEEXT_max)
     	pnew += count - FILEEXT_max;
     strcpy (option_file_ext, pnew);
	}

long Initialize_Gathering (void) {
RECORD_File_Start
		start;
struct tm gm_time;
struct utsname
		name;
long		val,
		gmt_seconds,
          local_seconds;
	if (file) fclose (file);
	Create_Intervals (Get_Now ());
     current_date = last_time;
     sprintf (file_name, "AIX%04d%02d%02d.%s", current_date.tm_year+1900,
     	current_date.tm_mon+1, current_date.tm_mday, option_file_ext);
	file = fopen (file_name, "a+b");
     if (! file)
     	abort ();
          	/* start record */
	val = RECID_group_start;
     fwrite (&val, sizeof (long), 1, file);
     memset (&start, 0x00, sizeof (RECORD_File_Start));
     start.size = sizeof (RECORD_File_Start);
     start.agent_version = AGENT_version;
     start.date = ((last_time.tm_year + 1900) * 10000) + ((last_time.tm_mon + 1) * 100) +
     	last_time.tm_mday;
     start.time = (last_time.tm_hour * 3600) + (last_time.tm_min * 60) + last_time.tm_sec;
     start.base_time = time (NULL);
	start.local_time = *localtime (&start.base_time);
	gm_time = *gmtime (&start.base_time);
     start.timezone_shift_seconds = Calculate_Time_Shift (gm_time, start.local_time);
     start.int_size = sizeof (int);
	start.endian_check = VALUE_endian;
     start.sizeof_fstab = sizeof (struct fstab);
     start.sizeof_lwpinfo = sizeof (struct lwpsinfo);
     start.sizeof_psinfo = sizeof (struct psinfo);
     start.sizeof_lwpstatus = sizeof (struct lwpstatus);
     start.sizeof_pstatus = sizeof (struct pstatus);
     start.sizeof_perfstat_cpu_total = sizeof (perfstat_cpu_total_t);
     start.sizeof_perfstat_cpu = sizeof (perfstat_cpu_t);
     start.sizeof_perfstat_disk = sizeof (perfstat_disk_t);
     start.sizeof_perfstat_memory_total = sizeof (perfstat_memory_total_t);
     start.sizeof_perfstat_netinterface = sizeof (perfstat_netinterface_t);
     strcpy (start.platform_type, "AIX");
     uname (&name);
     start.system_name_size = strlen (name.sysname) + 1;
     start.node_name_size = strlen (name.nodename) + 1;
     start.release_size = strlen (name.release) + 1;
     start.version_size = strlen (name.version) + 1;
     start.machine_size = strlen (name.machine) + 1;
     fwrite (&start, sizeof (RECORD_File_Start), 1, file);
     fwrite (name.sysname, start.system_name_size, 1, file);
     fwrite (name.nodename, start.node_name_size, 1, file);
     fwrite (name.release, start.release_size, 1, file);
     fwrite (name.version, start.version_size, 1, file);
     fwrite (name.machine, start.machine_size, 1, file);
	val = RECID_group_end;
     fwrite (&val, sizeof (long), 1, file);
     Store_Baseline_Information ();
     Set_Timer ();
     return (1);
     }

long Start_Collecting (void) {
	file = 0;
	next_is_review = 0;
	option_interval_minutes = 10;
     option_review_seconds = 5;
     strcpy (option_midnight_command, "midnight_file_transfer");
     Create_File_Extension ();
     Read_Options ();
     last_time = Get_Now ();
	Initialize_Process_Table ();
     if (! Initialize_Gathering ()) return (0);
     timer_interval = 1;
     return (1);
     }


/**************************************************************************/
/* 	Collecting Functions
*/

void Start_Command_File (void) {
FILE		*shell_file;
char		shell_command[500];
	if (! *option_midnight_command) return;
     shell_file = fopen (option_midnight_command, "r");
     if (! shell_file) return;
     fclose (shell_file);
     sprintf (shell_command, "%s %s", option_midnight_command, file_name);
	system (shell_command);
	}

void Collect_Information (void) {
struct tm time;
long		midnight_rollover,
		options_changed;
	FUNCTION_sleep (timer_interval);
     time = Get_Now ();
     midnight_rollover = (Get_Julian_Date (time) > Get_Julian_Date (last_time));
     last_time = time;
     if (midnight_rollover) {
		Store_Information (INFOTYPE_interval_record);
		if (midnight_rollover) {
               fclose (file);
               file = 0;
               Start_Command_File ();
               }
          next_is_review = 0;
          Initialize_Gathering ();
		}
        else {
		Store_Information (
          	(next_is_review) ? INFOTYPE_differ_record : INFOTYPE_interval_record);
          }     
	if (! next_is_review)
	     Read_Options ();
     Set_Timer ();
	}


/**************************************************************************/
/* 	Ending Functions
*/

long End_Collecting (void) {
	Store_Information (INFOTYPE_interval_record);
	End_Process_Table ();
     if (file) fclose (file);
     return (1);
	}


/**************************************************************************/
/* 	Main Function
*/

int main (int argc, char* argv[]) {
	directory = opendir ("/proc");
     if (! directory)
          return (100);
	diag = fopen ("diagdata", "wb");
	Start_Collecting ();
     fclose (diag);
/*	End_Collecting ();
	End_Disk_Map ();
     closedir (directory);
	return (0);
*/
	}

